#include <fxsaber\SingleTesterCache\SingleTesterCache.mqh> // https://www.mql5.com/ru/code/27611
#include <fxsaber\ThirdPartyTicks\File.mqh> // https://www.mql5.com/ru/code/20225

#include <Graphics\Graphic.mqh>

class VALIDATEBALANCE
{
private:
  double Balance[];

  static int GetPortfolio( const string &FileNames[], SINGLETESTERCACHE &Portfolio[], const int Common_Flag = 0 )
  {
    int Amount = 0;
    const int Size = ::ArrayResize(Portfolio, ::ArraySize(FileNames));

    for (int i = 0; i < Size; i++)
      if (Portfolio[Amount].Load(FileNames[i], Common_Flag))
        Amount++;

    return(::ArrayResize(Portfolio, Amount));
  }

#define MIN_WIDTH 10
  static string GraphPlot( const double &Y[], const long chartID = 0, int Width = 0, int Height = 0,
                           const ENUM_CURVE_TYPE Type = CURVE_NONE,
                           const string CurveName = NULL, string ObjName = NULL )
  {
    Width = Width ? Width : (int)::ChartGetInteger(chartID, CHART_WIDTH_IN_PIXELS);
    Height = Height ? Height : (int)::ChartGetInteger(chartID, CHART_HEIGHT_IN_PIXELS);
    ObjName = (ObjName == NULL) ? __FUNCTION__ : ObjName;

    CGraphic Graphic;

    const bool Res = (::ObjectFind(chartID, ObjName) >= 0) ? Graphic.Attach(chartID, ObjName) : Graphic.Create(chartID, ObjName, 0, 0, 0, Width, Height);

    if (Res)
    {
      const int Size = ::ArraySize(Y);

      Graphic.CurveAdd(Y, ((Type == CURVE_NONE) && Size) ? ((Width / Size < MIN_WIDTH) ? CURVE_LINES : CURVE_POINTS_AND_LINES) : Type, CurveName);

      Graphic.CurvePlotAll();
      Graphic.Update();
    }

    return (Res ? Graphic.ChartObjectName() : NULL);
  }
#undef MIN_WIDTH

  //  Bitmap-  bmp/gif/png- (  )
  static bool BitmapObjectToFile( const long chartID, const string ObjName, const string FileName, const bool FullImage = false )
  {
    const ENUM_OBJECT Type = (ENUM_OBJECT)ObjectGetInteger(chartID, ObjName, OBJPROP_TYPE);
    bool Res = (Type == OBJ_BITMAP_LABEL) || (Type == OBJ_BITMAP);

    if (Res)
    {
      const string Name = __FUNCTION__ + (string)MathRand();

      ObjectCreate(chartID, Name, OBJ_CHART, 0, 0, 0);
      ObjectSetInteger(chartID, Name, OBJPROP_XDISTANCE, -5e3);

      const long chart = ObjectGetInteger(chartID, Name, OBJPROP_CHART_ID);

      Res = ChartSetInteger(chart, CHART_SHOW, false) && ObjectCreate(chart, Name, OBJ_BITMAP_LABEL, 0, 0, 0) &&
            ObjectSetString(chart, Name, OBJPROP_BMPFILE, ObjectGetString(chartID, ObjName, OBJPROP_BMPFILE)) &&
            (FullImage || (ObjectSetInteger(chart, Name, OBJPROP_XSIZE, ObjectGetInteger(chartID, ObjName, OBJPROP_XSIZE)) &&
                           ObjectSetInteger(chart, Name, OBJPROP_YSIZE, ObjectGetInteger(chartID, ObjName, OBJPROP_YSIZE)) &&
                           ObjectSetInteger(chart, Name, OBJPROP_XOFFSET, ObjectGetInteger(chartID, ObjName, OBJPROP_XOFFSET)) &&
                           ObjectSetInteger(chart, Name, OBJPROP_YOFFSET, ObjectGetInteger(chartID, ObjName, OBJPROP_YOFFSET)))) &&
                           ChartScreenShot(chart, FileName, (int)ObjectGetInteger(chart, Name, OBJPROP_XSIZE),
                                                            (int)ObjectGetInteger(chart, Name, OBJPROP_YSIZE));
      ObjectDelete(chartID, Name);
    }

    return(Res);
  }

public:
  int Load( const string FolderName )
  {
    string FileNames[];
    SINGLETESTERCACHE SingleTesterCache[];

    ::ArrayFree(this.Balance);

    if (FILE::GetFileNames(FileNames, FolderName + "*.tst") &&
        VALIDATEBALANCE::GetPortfolio(FileNames, SingleTesterCache))
    {
      const int Size = ArraySize(SingleTesterCache);

      int MaxAmount = 0;

      for (int i = 0; i < Size; i++)
        MaxAmount += ::ArraySize(SingleTesterCache[i].Deals);

      double Profit[][2];

      ::ArrayResize(Profit, MaxAmount - Size);

      int Amount = 0;

      datetime MinDateFrom = INT_MAX;
      double BalanceBegin = 0;

      for (int i = 0; i < Size; i++)
      {
        if (SingleTesterCache[i].Header.date_from < MinDateFrom)
        {
          BalanceBegin = SingleTesterCache[i].Header.trade_deposit;

          MinDateFrom = SingleTesterCache[i].Header.date_from;
        }

        const int Size2 = ::ArraySize(SingleTesterCache[i].Deals);

        for (int j = 1; j < Size2; j++)
        {
          const TradeDeal Deal = SingleTesterCache[i].Deals[j];

          Profit[Amount][0] = (double)Deal.GetProperty(DEAL_TIME);
          Profit[Amount][1] = Deal.GetProperty(DEAL_PROFIT) + Deal.GetProperty(DEAL_SWAP) + Deal.GetProperty(DEAL_COMMISSION);

          Amount++;
        }
      }

      ::ArraySort(Profit);

      ::ArrayResize(this.Balance, Amount + 1);
      Balance[0] = BalanceBegin;

      for (int i = 0; i < Amount; i++)
        Balance[i + 1] = Balance[i] + Profit[i][1];
    }

    return(this.GetAmount());
  }

  void Visual( void ) const
  {
    VALIDATEBALANCE::GraphPlot(this.Balance, 0, 0, 0, CURVE_NONE, "Balance", __FUNCTION__);

    return;
  }

  void SaveVisual( const string FileName ) const
  {
    const string Name = __FUNCTION__ + (string)::MathRand();

    ::ObjectCreate(0, Name, OBJ_CHART, 0, 0, 0);
    ::ObjectSetInteger(0, Name, OBJPROP_XDISTANCE, -5e3);

    const long chart = ::ObjectGetInteger(0, Name, OBJPROP_CHART_ID);
    const string ObjName = VALIDATEBALANCE::GraphPlot(Balance, chart, ::TerminalInfoInteger(TERMINAL_SCREEN_WIDTH),
                                                      ::TerminalInfoInteger(TERMINAL_SCREEN_HEIGHT), CURVE_NONE, "Balance", __FUNCTION__);
    VALIDATEBALANCE::BitmapObjectToFile(chart, ObjName, FileName);

    ::ObjectDelete(0, Name);

    return;
  }

  int GetAmount( void ) const
  {
    return(::ArraySize(this.Balance));
  }

  double GetLastBalance( void ) const
  {
    return(this.GetAmount() ? this.Balance[this.GetAmount() - 1] : 0);
  }
};